/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2007 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "wallRadiationTemperatureFvPatchScalarField.H"
#include "RASModel.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "addToRunTimeSelectionTable.H"
#include "wallFvPatch.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace compressible
{
namespace RASModels
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

wallRadiationTemperatureFvPatchScalarField::wallRadiationTemperatureFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedValueFvPatchField<scalar>(p, iF),
    overallHeatTransferCoefficient_(p.size(), 0.0),
    outT_(p.size(), 0.0),
    QrName_("undefined-Qr"),
    relax_(p.size(), 0.0),
    value_(p.size(), 0.0)
{
}


wallRadiationTemperatureFvPatchScalarField::wallRadiationTemperatureFvPatchScalarField
(
    const wallRadiationTemperatureFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchField<scalar>(ptf, p, iF, mapper),
    overallHeatTransferCoefficient_(ptf.overallHeatTransferCoefficient_),
    outT_(ptf.outT_),
    QrName_(ptf.QrName_),
    relax_(ptf.relax_),
    value_(ptf.value_)
{}


wallRadiationTemperatureFvPatchScalarField::wallRadiationTemperatureFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<scalar>(p, iF, dict),
    overallHeatTransferCoefficient_("overallHeatTransferCoefficient", dict, p.size()),
    outT_("outT", dict, p.size()),
    QrName_(dict.lookup("Qr")),
    relax_("relaxationFactor", dict, p.size())
{
   /* if (dict.found("value"))
    {
        fvPatchScalarField::operator=
        (
            scalarField("value", dict, p.size())
        );
    }

    if (dict.found("gradient"))
    {
        gradient() = scalarField("gradient", dict, p.size());
        fixedGradientFvPatchScalarField::updateCoeffs();
        fixedGradientFvPatchScalarField::evaluate();
    }
    else
    {
        fvPatchField<scalar>::operator=(patchInternalField());
        gradient() = 0.0;
    }*/
}

wallRadiationTemperatureFvPatchScalarField::wallRadiationTemperatureFvPatchScalarField
(
    const wallRadiationTemperatureFvPatchScalarField& ptf
)
:
    fixedValueFvPatchField<scalar>(ptf),
    overallHeatTransferCoefficient_(ptf.overallHeatTransferCoefficient_),
    outT_(ptf.outT_),
    QrName_(ptf.QrName_),
    relax_(ptf.relax_),
    value_(ptf.value_)
{}


wallRadiationTemperatureFvPatchScalarField::wallRadiationTemperatureFvPatchScalarField
(
    const wallRadiationTemperatureFvPatchScalarField& ptf,
    const DimensionedField<scalar, volMesh>& iF
)
: 
    fixedValueFvPatchField<scalar>(ptf, iF),
    overallHeatTransferCoefficient_(ptf.overallHeatTransferCoefficient_),
    outT_(ptf.outT_),
    QrName_(ptf.QrName_),
    relax_(ptf.relax_),
    value_(ptf.value_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

// Update the coefficients associated with the patch field
void wallRadiationTemperatureFvPatchScalarField::updateCoeffs()
{

    if (updated())
    {
        return;
    }

    const label patchI = patch().index();

    const RASModel& rasModel = db().lookupObject<RASModel>("RASProperties");
    const scalarField& y = rasModel.y()[patchI];
    const scalarField alphaEffB_ = rasModel.alphaEff()().boundaryField()[patchI];
 //   const scalarField& rhow = rasModel.rho().boundaryField()[patchI];
    const scalarField& T = rasModel.thermo().T();
    const scalarField& Tw = rasModel.thermo().T().boundaryField()[patchI];
    const scalarField Cpw(rasModel.thermo().Cp(Tw, patchI));

    scalarField& TB = *this;
    scalarField TBold(Tw.size(), 0.0);
    scalarField TBnew(Tw.size(), 0.0);


    scalarField Qr(Tw.size(), 0.0);
    if (QrName_ != "none")
    {
        Qr = patch().lookupPatchField<volScalarField, scalar>(QrName_);
    }

    forAll(TB, faceI)
    {
        label faceCellI = patch().faceCells()[faceI];

        TBold[faceI]  = TB[faceI];

        TBnew[faceI]  =( overallHeatTransferCoefficient_[faceI] * outT_[faceI]
                    + alphaEffB_[faceI] *Cpw[faceI] * T[faceCellI] / y[faceI]
                    + Qr[faceI] ) 
                    /(alphaEffB_[faceI] *Cpw[faceI] / y[faceI] +overallHeatTransferCoefficient_[faceI]);

        TB[faceI]     = TBold[faceI]  + relax_[faceI] * (TBnew[faceI] - TBold[faceI]);
    }
    fixedValueFvPatchField<scalar>::updateCoeffs();

}


// Write
void wallRadiationTemperatureFvPatchScalarField::write(Ostream& os) const
{
    fvPatchScalarField::write(os);
    overallHeatTransferCoefficient_.writeEntry("overallHeatTransferCoefficient", os);
    outT_.writeEntry("outT", os);
    os.writeKeyword("Qr")<< QrName_ << token::END_STATEMENT << nl;
    relax_.writeEntry("relaxationFactor", os);
//    radiateHeat_.writeEntry("radiateHeat", os);
    this->writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchScalarField, wallRadiationTemperatureFvPatchScalarField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace RASModels
} // End namespace compressible
} // End namespace Foam

// ************************************************************************* //
